﻿using System.Linq;
using Carmenta.Engine;

namespace NauticalWaypointsAndEta
{
    public class Routing
    {
        private readonly Operator _terrainRouteOperator;
        private readonly MemoryDataSet _waypointsDataSet;

        private readonly Atom _accumulatedTime = new Atom("accumulatedTimeSeconds");
        private readonly Atom _accumulatedTimeString = new Atom("accumulatedTimeString");

        public Routing()
        {
            Configuration routingConfig = new Configuration("bathymetry_routing.px");

            _terrainRouteOperator = routingConfig.GetPublicObject("TerrainRouteOperator0") as Operator;

            if (_terrainRouteOperator == null) return;

            _waypointsDataSet = _terrainRouteOperator.FindChildObject("WaypointsMemoryDataSet") as MemoryDataSet;
        }

        public FeatureCollection GetItinerary(out double eta, out string etaString)
        {
            eta = -1;
            etaString = "?";

            FeatureCollection itinerary = new FeatureCollection();

            if (_terrainRouteOperator == null) return itinerary;

            // Technically, you should be using the "itineraryDataSet.Crs" here, but in our example they are the same.
            var output = _terrainRouteOperator.GetFeatures(_waypointsDataSet.Crs).ToList();
            itinerary.AddRange(output);

            // Get the last leg which has a valid time.
            var lastValidLeg = itinerary.LastOrDefault(leg =>
                leg.Attributes.ContainsKey(_accumulatedTime) &&
                leg.Attributes.ContainsKey(_accumulatedTimeString) &&
                leg.Attributes[_accumulatedTimeString].TryGetValue(out string timeString) &&
                timeString != "?" && timeString != "?");

            // No valid leg. Return.
            if (lastValidLeg == null) return itinerary;

            // Get the ETA from the last valid leg.
            lastValidLeg.Attributes[_accumulatedTime].TryGetValue(out eta);
            lastValidLeg.Attributes[_accumulatedTimeString].TryGetValue(out etaString);
            return itinerary;
        }
    }
}
